@@ -1,6 +1,8 @@
+from collections import defaultdict
 from enum import Enum
 from functools import lru_cache
-from typing import Type, List, cast, Optional
+from parser import ParserError
+from typing import Type, List, cast, Optional, Dict
 
 import pendulum
 from duckling import (load_time_zones, parse_ref_time,
@@ -11,6 +13,8 @@ from pymultirole_plugins.util import comma_separated_to_list
 from pymultirole_plugins.v1.annotator import AnnotatorParameters, AnnotatorBase
 from pymultirole_plugins.v1.schema import Document, Span, Annotation
 
+DEFAULT_DIM_LABEL = "Dimension"
+
 
 class Dimension(str, Enum):
     amount_of_money = 'amount-of-money'
@@ -27,12 +31,37 @@ class Dimension(str, Enum):
     volume = 'volume'
 
 
+SUPPORTED_LANGUAGES = "en,fr,de,nl,es,pt,it,zh,ja,ru,hi"
+
+
 class DucklingParameters(AnnotatorParameters):
+    mapping: Dict[str, List[Dimension]] = Field(
+        None,
+        description="""Map a label to a list of [Duckling](https://github.com/facebook/duckling) dimensions to consider, to be chosen among:<br/>
+        <li>`amount-of-money`: Measures an amount of money such as *$20*, *30 euros*.
+        <li>`credit-card-number`: Captures a credit card number.
+        <li>`distance`: Captures a distance in miles or kilometers such as *5km*, *5 miles* and *12m*.
+        <li>`duration`: Captures a duration such as *30min*, *2 hours* or *15sec* and normalizes the value in seconds.
+        <li>`email`: Captures an email but do not try to check the validity of the email. For example, *support@kairntech.com*.
+        <li>`number`: Extrapolates a number from free text, such as *six*,*twelve*, *16*, *1.10* and *23K*.
+        <li>`ordinal`: Captures the measure of an ordinal number, such as *first*, *second*, *third*... or *1st*, *2nd*, ..., *7th*, etc.
+        <li>`phone-number`: Captures phone numbers, but does not try to check the validity of the number.
+        <li>`quantity`: Captures the quantity of something; such as ingredients in recipes, or quantities of food for health tracking apps. Returns a numerical value, a unit, and a product (each field is optional).
+        <li>`temperature`: Captures the temperature in units of celsius or fahrenheit degrees.
+        <li>`time`: Captures and resolves date and time, like *tomorrow at 6pm*.
+        <li>`url`: Captures an URL, but does not try to check the validity of the URL.
+        <li>`volume`: Captures measures of volume like *250 ml*, *3 gal*.
+        If no mapping is provided a default label `Dimension` will be used for all dimensions.
+        """,
+        extra="key:label",
+    )
     time_zone: str = Field("Europe/Paris",
-                           description="Reference time zone to normalize date/time, must be a valid [time zone](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones)")
-    lang: str = Field("fr", description="Reference language")
-    locale: Optional[str] = Field(None, description="Reference locale, default is the default locale of the language")
-    dimensions: Optional[str] = Field("amount-of-money,time",
+                           description="Reference time zone to normalize date/time, must be a valid [time zone](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones)",
+                           extra="advanced"
+                           )
+    locale: Optional[str] = Field(None, description="Reference locale, default is the default locale of the language",
+                                  extra="advanced")
+    dimensions: Optional[str] = Field(None,
                                       description="""Comma-separated list of [Duckling](https://github.com/facebook/duckling) dimensions to consider, to be chosen among:<br/>
         <li>`amount-of-money`: Measures an amount of money such as *$20*, *30 euros*.
         <li>`credit-card-number`: Captures a credit card number.
@@ -47,7 +76,7 @@ class DucklingParameters(AnnotatorParameters):
         <li>`time`: Captures and resolves date and time, like *tomorrow at 6pm*.
         <li>`url`: Captures an URL, but does not try to check the validity of the URL.
         <li>`volume`: Captures measures of volume like *250 ml*, *3 gal*.
-        """)
+        """, extra="internal")
 
 
 time_zones = load_time_zones("/usr/share/zoneinfo")
@@ -55,28 +84,37 @@ time_zones = load_time_zones("/usr/share/zoneinfo")
 
 class DucklingAnnotator(AnnotatorBase):
     """[Duckling](https://github.com/facebook/duckling) annotator.
-    """
+    #need-segments
+    #languages:""" + SUPPORTED_LANGUAGES
 
     def annotate(self, documents: List[Document], parameters: AnnotatorParameters) \
             -> List[Document]:
         params: DucklingParameters = \
             cast(DucklingParameters, parameters)
-        # Create parsing context with time and language information
-        context = get_context(params.time_zone, params.lang, params.locale)
 
-        # Define dimensions to look-up for, check if they are valid names
-        valid_dimensions = []
-        for d in comma_separated_to_list(params.dimensions):
-            try:
-                Dimension(d)
-                valid_dimensions.append(d)
-            except ValueError:
-                pass
+        supported_languages = comma_separated_to_list(SUPPORTED_LANGUAGES)
+        if params.mapping is not None:
+            duckling2labels = defaultdict(list)
+            for k, dim_lst in params.mapping.items():
+                for dim in dim_lst:
+                    dim = dim.value if isinstance(dim, Dimension) else dim
+                    duckling2labels[dim].append(k)
+            valid_dimensions = list(duckling2labels.keys())
+        else:
+            duckling2labels = None
+            valid_dimensions = [d.value for d in Dimension]
 
         # Parse dimensions to use
         output_dims = parse_dimensions(valid_dimensions)
 
         for document in documents:
+            lang = document_language(document, None)
+            if lang is None or lang not in supported_languages:
+                raise AttributeError(f"Metadata language {lang} is required and must be in {SUPPORTED_LANGUAGES}")
+
+            # Create parsing context with time and language information
+            context = get_context(params.time_zone, lang, params.locale)
+
             document.annotations = []
             if not document.sentences:
                 document.sentences = [Span(start=0, end=len(document.text))]
@@ -84,14 +122,29 @@ class DucklingAnnotator(AnnotatorBase):
                 # Parse a phrase
                 dims = parse(document.text[sent.start:sent.end], context, output_dims, False)
                 for dim in dims:
+                    dname = dim['dim']
                     start = sent.start + dim['start']
                     end = sent.start + dim['end']
                     props = dim.get('value', {})
                     props.pop('values', None)
-                    document.annotations.append(Annotation(start=start, end=end,
-                                                           labelName=dim['dim'].replace('-', '_'), label=dim['dim'],
-                                                           text=document.text[start:end],
-                                                           properties=props))
+                    props['dim'] = dname
+                    if dname == "time":
+                        try:
+                            dt = pendulum.parse(props['value'])
+                            props['value'] = dt.to_iso8601_string()
+                        except ParserError:
+                            pass
+                    if duckling2labels is None:
+                        document.annotations.append(Annotation(start=start, end=end,
+                                                               label=DEFAULT_DIM_LABEL,
+                                                               text=document.text[start:end],
+                                                               properties=props))
+                    elif dname in duckling2labels:
+                        for lname in duckling2labels[dname]:
+                            document.annotations.append(Annotation(start=start, end=end,
+                                                                   labelName=lname,
+                                                                   text=document.text[start:end],
+                                                                   properties=props))
         return documents
 
     @classmethod
@@ -99,6 +152,12 @@ class DucklingAnnotator(AnnotatorBase):
         return DucklingParameters
 
 
+def document_language(doc: Document, default: str = None):
+    if doc.metadata is not None and 'language' in doc.metadata:
+        return doc.metadata['language']
+    return default
+
+
 @lru_cache(maxsize=None)
 def get_context(time_zone, lang, locale):
     bog_now = pendulum.now(time_zone).replace(microsecond=0)
